@@ -10,10 +10,13 @@ from wtforms.fields import (
     TextField,
     _unset_value
 )
+from wtforms.ext.sqlalchemy.fields import (
+    QuerySelectField, QuerySelectMultipleField
+)
 from wtforms.validators import Optional, DataRequired
 
 
-__version__ = '0.2.2'
+__version__ = '0.2.3'
 
 
 class InvalidData(Exception):
@@ -128,15 +131,33 @@ def patch_data(self):
 
         if isinstance(f, FormField):
             data[name] = f.patch_data
+        elif (isinstance(f, FieldList) and
+                f.unbound_field.field_class.__name__ == 'FormField'):
+            data[name] = [i.patch_data for i in f.entries]
         else:
             data[name] = f.data
     return data
 
 
-def monkey_patch_process(func):
+def monkey_patch_form_process(func):
     """
     Monkey patches Form.process method to better understand missing values.
     """
+    def process(self, formdata, data=_unset_value, original_keys=[], **kwargs):
+        if hasattr(self, '_original_keys'):
+            original_keys = self._original_keys
+        for name, field in self._fields.iteritems():
+            field._is_missing = (name not in original_keys)
+        self._original_keys = original_keys
+        func(self, formdata, data=data, **kwargs)
+
+    return process
+
+
+def monkey_patch_field_process(func):
+    """
+    Monkey patches Field.process method to better understand missing values.
+    """
     def process(self, formdata, data=_unset_value):
         call_original_func = True
         if not isinstance(self, FormField):
@@ -152,6 +173,7 @@ def monkey_patch_process(func):
                     self.is_missing = not bool(formdata.getlist(self.name))
                 else:
                     self.is_missing = True
+
         if call_original_func:
             func(self, formdata, data=data)
 
@@ -168,6 +190,7 @@ def monkey_patch_process(func):
                 self.data = u''
             else:
                 self.data = six.text_type(self.data)
+
     return process
 
 
@@ -184,7 +207,14 @@ class MultiDict(dict):
 
 @classmethod
 def from_json(cls, formdata=None, obj=None, **kwargs):
-    return cls(MultiDict(flatten_json(cls, formdata)), obj, **kwargs)
+    original_keys = formdata.keys() if formdata else []
+    form = cls(
+        MultiDict(flatten_json(cls, formdata)),
+        obj,
+        original_keys=original_keys,
+        **kwargs
+    )
+    return form
 
 
 def boolean_process_formdata(self, valuelist):
@@ -209,14 +239,33 @@ def is_missing(self):
 
 @property
 def field_list_is_missing(self):
+    if hasattr(self, '_is_missing'):
+        return self._is_missing
+
     return all([field.is_missing for field in self.entries])
 
 
+def monkey_patch_process_formdata(func):
+    def process_formdata(self, valuelist):
+        valuelist = map(six.text_type, valuelist)
+
+        return func(self, valuelist)
+    return process_formdata
+
+
 def init():
     Form.is_missing = is_missing
-    Form.patch_data = patch_data
-    Form.from_json = from_json
     FieldList.is_missing = field_list_is_missing
-    Field.process = monkey_patch_process(Field.process)
-    FormField.process = monkey_patch_process(FormField.process)
+    Form.from_json = from_json
+    Form.patch_data = patch_data
+    FieldList.patch_data = patch_data
+    Form.process = monkey_patch_form_process(Form.process)
+    QuerySelectField.process_formdata = monkey_patch_process_formdata(
+        QuerySelectField.process_formdata
+    )
+    QuerySelectMultipleField.process_formdata = monkey_patch_process_formdata(
+        QuerySelectMultipleField.process_formdata
+    )
+    Field.process = monkey_patch_field_process(Field.process)
+    FormField.process = monkey_patch_field_process(FormField.process)
     BooleanField.process_formdata = boolean_process_formdata
